@@ -24,7 +24,7 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 # OTHER DEALINGS IN THE SOFTWARE.
 from threading import local
-version = '0.1.3-unreleased'
+version = '0.1.5-unreleased'
 
 def that_with_context(setup=None, teardown=None):
     def dec(func):
@@ -75,7 +75,12 @@ def all_integers(obj):
     return True
 
 class that(object):
-    def __init__(self, src, within_range=None):
+    def __init__(self, src,
+                 within_range=None,
+                 with_args=None,
+                 with_kwargs=None,
+                 and_kwargs=None):
+
         self._src = src
         self._attribute = None
         self._eval = None
@@ -88,6 +93,32 @@ class that(object):
 
             self._range = within_range
 
+        self._callable_args = []
+        self._callable_kw = {}
+        if isinstance(with_kwargs, dict):
+            self._callable_kw.update(with_kwargs)
+
+        if isinstance(and_kwargs, dict):
+            self._callable_kw.update(and_kwargs)
+
+    def raises(self, exc, msg=None):
+        if not callable(self._src):
+            raise TypeError('%r is not callable' % self._src)
+
+        try:
+            self._src(*self._callable_args, **self._callable_kw)
+        except Exception, e:
+            if isinstance(exc, type) and issubclass(exc, Exception):
+                if not isinstance(e, exc):
+                    raise AssertionError('%r should raise %r, but raised %r' % (self._src, exc, e.__class__))
+
+                if isinstance(msg, basestring) and msg != unicode(e):
+                    raise AssertionError('%r raised %s, but the exception message does not match. Expected %r, got %r' % (self._src, e, msg, e))
+            elif isinstance(msg, basestring) and msg != unicode(e):
+                raise AssertionError('When calling %r the exception message does not match. Expected %s, got %s' % (self._src, msg, e))
+
+        return True
+
     def equals(self, dst):
         if self._attribute and is_iterable(self._src):
             msg = '%r[%d].%s should be %r, but is %r'
@@ -164,20 +195,35 @@ class that(object):
 
     def matches(self, items):
         msg = '%r[%d].%s should be %r, but is %r'
+        get_eval = lambda item: eval(
+            "%s.%s" % ('current', self._eval), {}, {'current': item}
+        )
+
         if self._eval and is_iterable(self._src):
             if isinstance(items, basestring):
                 items = [items for x in range(len(items))]
+            else:
+                if len(items) != len(self._src):
+                    source = map(get_eval, self._src)
+                    source_len = len(source)
+                    items_len = len(items)
+
+                    raise AssertionError(
+                        '%r has %d items, but the matching list has %d: %r'
+                        % (source, source_len, items_len, items)
+                    )
 
             for index, (item, other) in enumerate(zip(self._src, items)):
                 if self._range:
                     if index < self._range[0] or index > self._range[1]:
                         continue
 
-                value = eval("%s.%s" % ('current', self._eval), {}, {'current': item})
+                value = get_eval(item)
 
                 error = msg % (self._src, index, self._eval, other, value)
                 if other != value:
                     raise AssertionError(error)
+
         return True
 
     def __contains__(self, what):
